@@ -32,6 +32,7 @@
 #include "common/open-utils.h"
 #include "cmds/commands.h"
 #include "common/utils.h"
+#include "common/parse-utils.h"
 #include "common/help.h"
 
 static const char * const balance_cmd_group_usage[] = {
@@ -42,30 +43,18 @@ static const char * const balance_cmd_group_usage[] = {
 
 static int parse_one_profile(const char *profile, u64 *flags)
 {
-	if (!strcmp(profile, "raid0")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID0;
-	} else if (!strcmp(profile, "raid1")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID1;
-	} else if (!strcmp(profile, "raid1c3")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID1C3;
-	} else if (!strcmp(profile, "raid1c4")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID1C4;
-	} else if (!strcmp(profile, "raid10")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID10;
-	} else if (!strcmp(profile, "raid5")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID5;
-	} else if (!strcmp(profile, "raid6")) {
-		*flags |= BTRFS_BLOCK_GROUP_RAID6;
-	} else if (!strcmp(profile, "dup")) {
-		*flags |= BTRFS_BLOCK_GROUP_DUP;
-	} else if (!strcmp(profile, "single")) {
-		*flags |= BTRFS_AVAIL_ALLOC_BIT_SINGLE;
-	} else {
+	int ret;
+	u64 tmp = 0;
+
+	ret = parse_bg_profile(profile, &tmp);
+	if (ret) {
 		error("unknown profile: %s", profile);
 		return 1;
 	}
-
-	return 0;
+	if (tmp == 0)
+		tmp = BTRFS_AVAIL_ALLOC_BIT_SINGLE;
+	*flags |= tmp;
+	return ret;
 }
 
 static int parse_profiles(char *profiles, u64 *flags)
@@ -83,122 +72,6 @@ static int parse_profiles(char *profiles, u64 *flags)
 	return 0;
 }
 
-static int parse_u64(const char *str, u64 *result)
-{
-	char *endptr;
-	u64 val;
-
-	val = strtoull(str, &endptr, 10);
-	if (*endptr)
-		return 1;
-
-	*result = val;
-	return 0;
-}
-
-/*
- * Parse range that's missing some part that can be implicit:
- * a..b	- exact range, a can be equal to b
- * a..	- implicitly unbounded maximum (end == (u64)-1)
- * ..b	- implicitly starting at 0
- * a	- invalid; unclear semantics, use parse_u64 instead
- *
- * Returned values are u64, value validation and interpretation should be done
- * by the caller.
- */
-static int parse_range(const char *range, u64 *start, u64 *end)
-{
-	char *dots;
-	char *endptr;
-	const char *rest;
-	int skipped = 0;
-
-	dots = strstr(range, "..");
-	if (!dots)
-		return 1;
-
-	rest = dots + 2;
-
-	if (!*rest) {
-		*end = (u64)-1;
-		skipped++;
-	} else {
-		*end = strtoull(rest, &endptr, 10);
-		if (*endptr)
-			return 1;
-	}
-	if (dots == range) {
-		*start = 0;
-		skipped++;
-	} else {
-		*start = strtoull(range, &endptr, 10);
-		if (*endptr != 0 && *endptr != '.')
-			return 1;
-	}
-
-	if (*start > *end) {
-		error("range %llu..%llu doesn't make sense",
-			(unsigned long long)*start,
-			(unsigned long long)*end);
-		return 1;
-	}
-
-	if (skipped <= 1)
-		return 0;
-
-	return 1;
-}
-
-/*
- * Parse range and check if start < end
- */
-static int parse_range_strict(const char *range, u64 *start, u64 *end)
-{
-	if (parse_range(range, start, end) == 0) {
-		if (*start >= *end) {
-			error("range %llu..%llu not allowed",
-				(unsigned long long)*start,
-				(unsigned long long)*end);
-			return 1;
-		}
-		return 0;
-	}
-
-	return 1;
-}
-
-/*
- * Convert 64bit range to 32bit with boundary checks
- */
-static int range_to_u32(u64 start, u64 end, u32 *start32, u32 *end32)
-{
-	if (start > (u32)-1)
-		return 1;
-
-	if (end != (u64)-1 && end > (u32)-1)
-		return 1;
-
-	*start32 = (u32)start;
-	*end32 = (u32)end;
-
-	return 0;
-}
-
-__attribute__ ((unused))
-static int parse_range_u32(const char *range, u32 *start, u32 *end)
-{
-	u64 tmp_start;
-	u64 tmp_end;
-
-	if (parse_range(range, &tmp_start, &tmp_end))
-		return 1;
-
-	if (range_to_u32(tmp_start, tmp_end, start, end))
-		return 1;
-
-	return 0;
-}
-
 __attribute__ ((unused))
 static void print_range(u64 start, u64 end)
 {
@@ -645,8 +518,7 @@ static int cmd_balance_start(const struct cmd_struct *cmd,
 		if (!(ptrs[i]->flags & BTRFS_BALANCE_ARGS_CONVERT))
 			continue;
 
-		if (!(ptrs[i]->target & (BTRFS_BLOCK_GROUP_RAID6 |
-					   BTRFS_BLOCK_GROUP_RAID5)))
+		if (!(ptrs[i]->target & BTRFS_BLOCK_GROUP_RAID56_MASK))
 			continue;
 
 		if (raid56_warned)
