@@ -46,7 +46,7 @@ static void print_extents(struct extent_buffer *eb)
 		return;
 
 	if (btrfs_is_leaf(eb)) {
-		btrfs_print_leaf(eb);
+		btrfs_print_leaf(eb, BTRFS_PRINT_TREE_DEFAULT);
 		return;
 	}
 
@@ -83,8 +83,14 @@ out:
 
 static void print_old_roots(struct btrfs_super_block *super)
 {
+	const char *extent_tree_str = "extent root";
 	struct btrfs_root_backup *backup;
 	int i;
+	bool extent_tree_v2 = (btrfs_super_incompat_flags(super) &
+		BTRFS_FEATURE_INCOMPAT_EXTENT_TREE_V2);
+
+	if (extent_tree_v2)
+		extent_tree_str = "block group root";
 
 	for (i = 0; i < BTRFS_NUM_BACKUP_ROOTS; i++) {
 		backup = super->super_roots + i;
@@ -93,7 +99,7 @@ static void print_old_roots(struct btrfs_super_block *super)
 		       (unsigned long long)btrfs_backup_tree_root_gen(backup),
 		       (unsigned long long)btrfs_backup_tree_root(backup));
 
-		printf("\t\textent root gen %llu block %llu\n",
+		printf("\t\t%s gen %llu block %llu\n", extent_tree_str,
 		       (unsigned long long)btrfs_backup_extent_root_gen(backup),
 		       (unsigned long long)btrfs_backup_extent_root(backup));
 
@@ -155,7 +161,8 @@ static u64 treeid_from_string(const char *str, const char **end)
 		{ "TREE_LOG_FIXUP", BTRFS_TREE_LOG_FIXUP_OBJECTID },
 		{ "TREE_LOG", BTRFS_TREE_LOG_OBJECTID },
 		{ "TREE_RELOC", BTRFS_TREE_RELOC_OBJECTID },
-		{ "DATA_RELOC", BTRFS_DATA_RELOC_TREE_OBJECTID }
+		{ "DATA_RELOC", BTRFS_DATA_RELOC_TREE_OBJECTID },
+		{ "BLOCK_GROUP_TREE", BTRFS_BLOCK_GROUP_TREE_OBJECTID },
 	};
 
 	if (strncasecmp("BTRFS_", str, strlen("BTRFS_")) == 0)
@@ -207,6 +214,8 @@ static const char * const cmd_inspect_dump_tree_usage[] = {
 	"--bfs                  breadth-first traversal of the trees, print nodes, then leaves (default)",
 	"--dfs                  depth-first traversal of the trees",
 	"--hide-names           hide filenames/subvolume/xattrs and other name references",
+	"--csum-headers         print node checksums stored in headers (metadata)",
+	"--csum-items           print checksums stored in checksum items (data)",
 	NULL
 };
 
@@ -250,7 +259,7 @@ static int dump_add_tree_block(struct cache_tree *tree, u64 bytenr)
  * error.
  */
 static int dump_print_tree_blocks(struct btrfs_fs_info *fs_info,
-				  struct cache_tree *tree, bool follow)
+				  struct cache_tree *tree, unsigned int mode)
 {
 	struct cache_extent *ce;
 	struct extent_buffer *eb;
@@ -280,7 +289,7 @@ static int dump_print_tree_blocks(struct btrfs_fs_info *fs_info,
 			ret = -EIO;
 			goto next;
 		}
-		btrfs_print_tree(eb, follow, BTRFS_PRINT_TREE_DEFAULT);
+		btrfs_print_tree(eb, mode);
 		free_extent_buffer(eb);
 next:
 		remove_cache_extent(tree, ce);
@@ -317,7 +326,9 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 	u64 block_bytenr;
 	struct btrfs_root *tree_root_scan;
 	u64 tree_id = 0;
-	bool follow = false;
+	unsigned int follow = 0;
+	unsigned int csum_mode = 0;
+	unsigned int print_mode;
 
 	/*
 	 * For debug-tree, we care nothing about extent tree (it's just backref
@@ -332,7 +343,9 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 	while (1) {
 		int c;
 		enum { GETOPT_VAL_FOLLOW = 256, GETOPT_VAL_DFS, GETOPT_VAL_BFS,
-		       GETOPT_VAL_NOSCAN, GETOPT_VAL_HIDE_NAMES };
+		       GETOPT_VAL_NOSCAN, GETOPT_VAL_HIDE_NAMES,
+		       GETOPT_VAL_CSUM_HEADERS, GETOPT_VAL_CSUM_ITEMS,
+		};
 		static const struct option long_options[] = {
 			{ "extents", no_argument, NULL, 'e'},
 			{ "device", no_argument, NULL, 'd'},
@@ -346,6 +359,8 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 			{ "dfs", no_argument, NULL, GETOPT_VAL_DFS },
 			{ "noscan", no_argument, NULL, GETOPT_VAL_NOSCAN },
 			{ "hide-names", no_argument, NULL, GETOPT_VAL_HIDE_NAMES },
+			{ "csum-headers", no_argument, NULL, GETOPT_VAL_CSUM_HEADERS },
+			{ "csum-items", no_argument, NULL, GETOPT_VAL_CSUM_ITEMS },
 			{ NULL, 0, NULL, 0 }
 		};
 
@@ -402,7 +417,7 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 			break;
 			}
 		case GETOPT_VAL_FOLLOW:
-			follow = true;
+			follow = BTRFS_PRINT_TREE_FOLLOW;
 			break;
 		case GETOPT_VAL_DFS:
 			traverse = BTRFS_PRINT_TREE_DFS;
@@ -416,6 +431,12 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 		case GETOPT_VAL_HIDE_NAMES:
 			open_ctree_flags |= OPEN_CTREE_HIDE_NAMES;
 			break;
+		case GETOPT_VAL_CSUM_HEADERS:
+			csum_mode |= BTRFS_PRINT_TREE_CSUM_HEADERS;
+			break;
+		case GETOPT_VAL_CSUM_ITEMS:
+			csum_mode |= BTRFS_PRINT_TREE_CSUM_ITEMS;
+			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -468,9 +489,11 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 		goto out;
 	}
 
+	print_mode = follow | traverse | csum_mode;
+
 	if (!cache_tree_empty(&block_root)) {
 		root = info->chunk_root;
-		ret = dump_print_tree_blocks(info, &block_root, follow);
+		ret = dump_print_tree_blocks(info, &block_root, print_mode);
 		goto close_root;
 	}
 
@@ -493,23 +516,34 @@ static int cmd_inspect_dump_tree(const struct cmd_struct *cmd,
 				       info->log_root_tree->node->start,
 					btrfs_header_level(
 						info->log_root_tree->node));
+			if (info->block_group_root)
+				printf("block group tree: %llu level %d\n",
+				       info->block_group_root->node->start,
+					btrfs_header_level(
+						info->block_group_root->node));
 		} else {
 			if (info->tree_root->node) {
 				printf("root tree\n");
-				btrfs_print_tree(info->tree_root->node, true,
-						 traverse);
+				btrfs_print_tree(info->tree_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 			}
 
 			if (info->chunk_root->node) {
 				printf("chunk tree\n");
-				btrfs_print_tree(info->chunk_root->node, true,
-						 traverse);
+				btrfs_print_tree(info->chunk_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 			}
 
 			if (info->log_root_tree) {
 				printf("log root tree\n");
 				btrfs_print_tree(info->log_root_tree->node,
-						 true, traverse);
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
+			}
+
+			if (info->block_group_root) {
+				printf("block group tree\n");
+				btrfs_print_tree(info->block_group_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 			}
 		}
 	}
@@ -529,7 +563,8 @@ again:
 			goto close_root;
 		}
 		printf("root tree\n");
-		btrfs_print_tree(info->tree_root->node, true, traverse);
+		btrfs_print_tree(info->tree_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 		goto close_root;
 	}
 
@@ -539,7 +574,8 @@ again:
 			goto close_root;
 		}
 		printf("chunk tree\n");
-		btrfs_print_tree(info->chunk_root->node, true, traverse);
+		btrfs_print_tree(info->chunk_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 		goto close_root;
 	}
 
@@ -549,7 +585,19 @@ again:
 			goto close_root;
 		}
 		printf("log root tree\n");
-		btrfs_print_tree(info->log_root_tree->node, true, traverse);
+		btrfs_print_tree(info->log_root_tree->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
+		goto close_root;
+	}
+
+	if (tree_id && tree_id == BTRFS_BLOCK_GROUP_TREE_OBJECTID) {
+		if (!info->block_group_root) {
+			error("cannot print block group tree, invalid pointer");
+			goto close_root;
+		}
+		printf("block group tree\n");
+		btrfs_print_tree(info->block_group_root->node,
+					BTRFS_PRINT_TREE_FOLLOW | print_mode);
 		goto close_root;
 	}
 
@@ -676,6 +724,10 @@ again:
 					printf("multiple");
 				}
 				break;
+			case BTRFS_BLOCK_GROUP_TREE_OBJECTID:
+				if (!skip)
+					printf("block group");
+				break;
 			default:
 				if (!skip) {
 					printf("file");
@@ -695,7 +747,8 @@ again:
 					       btrfs_header_level(buf));
 				} else {
 					printf(" \n");
-					btrfs_print_tree(buf, true, traverse);
+					btrfs_print_tree(buf,
+						BTRFS_PRINT_TREE_FOLLOW | print_mode);
 				}
 			}
 			free_extent_buffer(buf);
