@@ -20,15 +20,7 @@
 #ifndef __BTRFS_SEND_H__
 #define __BTRFS_SEND_H__
 
-#if BTRFS_FLAT_INCLUDES
 #include "kernel-shared/ctree.h"
-#else
-#include <btrfs/ctree.h>
-#endif /* BTRFS_FLAT_INCLUDES */
-
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #define BTRFS_SEND_STREAM_MAGIC "btrfs-stream"
 #define BTRFS_SEND_STREAM_VERSION 1
@@ -140,8 +132,4 @@ enum {
 };
 #define BTRFS_SEND_A_MAX (__BTRFS_SEND_A_MAX - 1)
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
